/**
 * @file
 * Contains definition of the behavior WOW.js.
 */

(function ($, Drupal, drupalSettings, once) {
  "use strict";

  // Load settings from drupalSettings.
  const compat = drupalSettings.wowjs.compat;
  const wClass = drupalSettings.wowjs.wowClass;

  Drupal.behaviors.wowJS = {
    attach: function (context, settings) {
      // Retrieve the elements to be animated.
      const elements = drupalSettings.wowjs.elements;

      $.each(elements, function (index, element) {
        // Define options for WOW.js and ensure defaults for once and mirror.
        let options = {
          selector: element.selector,
          animation: element.animation,
          delay: element.delay,
          time: element.time,
          speed: element.speed,
          duration: element.duration,
          repeat: element.repeat,
          display: element.display,
          clean: element.clean,
          classBox: wClass,
          offset: element.wow.offset,
          mobile: element.wow.mobile,
          live: element.wow.live,
          once: element.wow.once !== undefined ? element.wow.once : true,
          mirror: element.wow.mirror !== undefined ? element.wow.mirror : false,
          optionalContainer: element.wow.optionalContainer,
          scrollContainer: element.wow.scrollContainer,
        };

        // Ensure WOW.js is initialized only once for each element.
        if (once('wowjs', options.selector).length) {
          new Drupal.WOW(options);
        }
      });
    }
  };

  /**
   * Drupal.WOW class definition.
   * Initializes WOW.js with the provided options.
   */
  Drupal.WOW = function (options) {
    // Remove previous animation classes if 'clean' is true.
    if (options.clean) {
      $(options.selector).removeClass(function (index, className) {
        return (className.match(/(^|\s)animate__\S+/g) || []).join(' ');
      });
    }

    let Prefix  = compat ? '' : 'animate__';
    let Classes = `${Prefix}animated ${Prefix}${options.animation}`;

    if (options.delay && options.delay != 'custom') {
      Classes += ` ${Prefix}${options.delay}`;
    }
    if (options.speed && options.speed != 'custom' && options.speed != 'medium') {
      Classes += ` ${Prefix}${options.speed}`;
    }
    if (options.repeat && options.repeat != 'repeat-1') {
      Classes += ` ${Prefix}${options.repeat}`;
    }

    // Add WOW box class to the selected elements.
    $(options.selector).addClass(options.classBox);

    let container = null;
    if (options.optionalContainer && options.scrollContainer !== 'window') {
      container = options.scrollContainer;
    }

    // Initialize WOW.js with the specified options.
    let wow = new WOW({
      boxClass: options.classBox,
      animateClass: Classes,
      offset: options.offset,
      mobile: options.mobile,
      live: options.live,
      callback: function (box) {
        // Callback function when an animation is started.
      },
      scrollContainer: container,
    });

    // Add WOW.js classes to the selected elements.
    $(options.selector).addClass(Classes);

    // Add custom properties for WOW.js animations.
    if (options.delay === 'custom' && !compat) {
      $(options.selector).css('--animate-delay', options.time);
    }
    if (options.speed === 'custom' && !compat) {
      $(options.selector).css('--animate-duration', options.duration);
    }

    // Add custom properties for Animate.css animations.
    if (options.delay === 'custom') {
      $(options.selector).css({
        '-webkit-animation-delay': `${options.time}ms`,
        '-moz-animation-delay': `${options.time}ms`,
        '-ms-animation-delay': `${options.time}ms`,
        '-o-animation-delay': `${options.time}ms`,
        'animation-delay': `${options.time}ms`,
        '--animate-delay': `${options.time}ms`,
      });
    }
    if (options.speed === 'custom') {
      $(options.selector).css({
        '-webkit-animation-duration': `${options.duration}ms`,
        '-moz-animation-duration': `${options.duration}ms`,
        '-ms-animation-duration': `${options.duration}ms`,
        '-o-animation-duration': `${options.duration}ms`,
        'animation-duration': `${options.duration}ms`,
        '--animate-duration': `${options.duration}ms`,
      });
    }

    // Check and change display property if required.
    if (options.display) {
      var displayValue = $(options.selector).css('display');
      if (displayValue === 'inline') {
        $(options.selector).css('display', 'inline-block');
      }
    }

    // Add event listener to manage animation on scroll.
    $(window).on('scroll', function (e) {
      const scrolled = $(window).scrollTop();
      const winHeightPadded = $(window).height() * 1.1;

      // Show elements in viewport
      $(`${options.selector}:not(.${Prefix}animated)`).each(function () {
        const $this = $(this);
        const offsetTop = $this.offset().top;

        if (scrolled + winHeightPadded > offsetTop) {
          if ($this.data('timeout')) {
            window.setTimeout(function (e) {
              $this.addClass(Classes).css('visibility', 'visible').css('animation-name', options.animation);
            }, parseInt($this.data('timeout'), 10));
          } else {
            $this.addClass(Classes).css('visibility', 'visible').css('animation-name', options.animation);
          }

          if (options.speed === 'custom') {
            $this.css('--animate-duration', options.duration);
          }

          if (options.delay === 'custom') {
            $this.css('--animate-delay', options.time);
          }
        }
      });

      // Hide elements out of viewport if mirror is true or once is false
      if (options.mirror || !options.once) {
        $(`${options.selector}.${Prefix}animated`).each(function () {
          const $this = $(this);
          const offsetTop = $this.offset().top;

          if (scrolled + winHeightPadded < offsetTop || (options.mirror && scrolled > offsetTop + $this.outerHeight())) {
            $this.removeClass(Classes).css('visibility', 'hidden').css('animation-name', 'none');
          }
        });
      }
    });

    // Initialize WOW.js.
    wow.init();
  };

})(jQuery, Drupal, drupalSettings, once);
